"""
Interfaces used by the Workspaces' products system.
"""

import inspect
from abc import ABC, abstractmethod
from enum import Enum
from pathlib import Path
from queue import PriorityQueue
from threading import Thread
from typing import Dict, List, Optional, Type

ProductLocator = str

"""

Products / Persistent Entities

"""


class CurrentArchiveProductIF(ABC, "ProductIF"):
    """
    Product that currently exists in the archive and can be located with a product locator
    """
    def __init__(self, product_locator: ProductLocator):
        self.product_locator = product_locator


"""

Products / Helpers

"""


class ProductIF(ABC):
    """
    Parent class for all science products
    """
    # FIXME: Needs attributes
    def __init__(self):
        pass


class FutureArchiveProductIF(ABC, ProductIF):
    """
    Class that represents a product that will be ingested into the archive after a capability
    execution has completed
    """
    # FIXME: Implement ProductType
    def __init__(self, future_product_type: "FutureProductTypeIF"):
        self.type = future_product_type


class FutureProductTypeIF(ABC):
    """
    Class that represents all possible types of future products
    """
    pass


class FutureExecutionBlockIF(ABC, FutureProductTypeIF):
    """
    Class that represents an EB that will be the result of a completed capability
    execution
    """
    pass


class FutureProductIF(ABC, FutureProductTypeIF):
    """
    Class that represents a product that will be the result of a completed capability execution
    """
    def __init__(self, product: Optional[ProductIF], product_type: Optional["ProductTypeIF"]):
        self.product = product
        self.product_type = product_type


class ProductTypeIF(ABC):
    """
    Class that represents all possible types of products that can result from a capability execution
    """
    # FIXME: We haven't decided how we want to implement this yet
    pass
